/**************************************************************************
* iptrkg- log grapher (iptrk - ip tracker)
* Copyright (C) 2003 Jason Nunn <jsno@leal.com.au>
* Darwin, NT, Australia.
* Originally developed for Arafura Connect (Dec 2000 - Mar 2003)
* Donated to GPL before liquidation.
*
* =======================================================================
* common routines for iptrkga and iptrkgt
*
**************************************************************************/
#include <gd.h>
#include "defs.h"
#include "lib.p"
#include "iptrkg_defs.h"

PUBLIC gdImagePtr  image;
PUBLIC int         height,
                   width = 800,
                   no_entries,
                   colour_black,
                   colour_white,
                   colour_red,
                   colour_green,
                   colour_blue,
                   colour_purple,
                   colour_cyan;
PUBLIC U_64        max_vol;
PUBLIC double      max_vol2;

/**************************************************************************
*
**************************************************************************/
PUBLIC void delete_old_pngs(void)
{
  char          str[STR_LEN + 1];
  DIR           *dp;
  struct dirent *d_st;

  printf("deleting old graphs.\n");

  dp = opendir(IPTRK_GRAPHS_PATH);
  if(dp == NULL)
  {
    log_e(__func__, "::opendir()");
    return;
  }

  for(;;)
  {
    d_st = readdir(dp);
    if(d_st == NULL)
      break;

    if(strstr(d_st->d_name, ".png") != NULL || strstr(d_st->d_name, ".html") != NULL)
    {
      sprintf(str, IPTRK_GRAPHS_PATH "/%s", d_st->d_name);
      unlink(str);
    }
  }

  closedir(dp);
}

/**************************************************************************
*
**************************************************************************/
PUBLIC void title_font(gdImagePtr i, int s, int x, int y, int c, char *text)
{
  char *err;

  err = gdImageStringFT(i, NULL, c, TTF_FONT, s, 0.0, x, y, text);

  if(err)
  {
    log_e(__func__, "::gdImageStringFT()");
    exit(-1);
  }
}

PUBLIC void title_font_align(
  gdImagePtr i, int s, int x, int y, int c, char *text, int align)
{
  char *err;
  int  brect[8], a;

  err = gdImageStringFT(NULL, &brect[0], c, TTF_FONT, s, 0.0, 0, 0, text);

  if(err)
  {
    log_e(__func__, "::gdImageStringFT()");
    exit(-1);
  }

  a = align - (MAXX(brect) - MINX(brect));
  if(a < 0)
    a = 0;
  gdImageStringFT(i, NULL, c, TTF_FONT, s, 0.0, a + x, y, text);
}

/**************************************************************************
*
**************************************************************************/
PUBLIC inline int scale_v(U_64 ss, U_64 ds, U_64 v)
{
  return (int)((v * ds) / ss);
}

/**************************************************************************
*
**************************************************************************/
PUBLIC void graph_init(void)
{
  U_64 b;
  int  i;
  char str[STR_LEN + 1];

  inline void draw_leg(char *l, int col)
  {
    gdImageFilledRectangle(image, i, 55, i + 10, 65, col);
    title_font(image, 14, i + 15, 65, col, l);
    i += 40;
  }

  b = max_vol % 10000;
  max_vol += 10000 - b;
  height = HIST_Y + 5 + (no_entries * BAR_SPACE);
  image = gdImageCreate(width, height);

  colour_white  = gdImageColorResolve(image, 255, 255, 255);
  colour_black  = gdImageColorResolve(image, 0, 0, 0);
  colour_red    = gdImageColorResolve(image, 255, 0, 0);
  colour_green  = gdImageColorResolve(image, 10, 180, 100);
  colour_blue   = gdImageColorResolve(image, 0, 0, 255);
  colour_purple = gdImageColorResolve(image, 255, 0, 255);
  colour_cyan   = gdImageColorResolve(image, 100, 100, 150);

  gdImageFilledRectangle(image, 0, 0, width, height, colour_black);

  sprintf(str, "(No of entries: %d)", no_entries);
  title_font_align(image, 15, width - 210, 20, colour_white, str, 200);
  title_font(image, 14, HIST_X, HIST_Y - 5, colour_white, "(Mb)");

  i = width - 190;
  draw_leg("I", colour_red);
  draw_leg("O", colour_blue);
  draw_leg("u", colour_purple);
  draw_leg("i", colour_cyan);
  draw_leg("o", colour_green);

  gdImageSetThickness(image, 3);
  gdImageLine(image, HIST_X, HIST_Y, width - 5, HIST_Y, colour_white);
  gdImageLine(image, HIST_X, HIST_Y, HIST_X, height - 5, colour_white);

  gdImageLine(
    image,
    BAR_START + BAR_SIZE, HIST_Y - 4,
    BAR_START + BAR_SIZE, HIST_Y + 2, colour_white);

  max_vol2 = (double)max_vol / 1000000; /*a meg isn't 2 ^ 20, it's 10 ^ 6*/
  sprintf(str, "%.02f", max_vol2);
  title_font_align(image, 14, width - 55, HIST_Y - 5, colour_white, str, 50);

  gdImageLine(
    image,
    BAR_START + (BAR_SIZE / 2), HIST_Y - 4,
    BAR_START + (BAR_SIZE / 2), HIST_Y + 2, colour_white);

  sprintf(str, "%.02f", max_vol2 / 2);
  title_font_align(
    image, 14, width - 57 - (BAR_SIZE / 2), HIST_Y - 5, colour_white, str, 50);
}
